/*
 * Example for Secure 3 click
 *
 * Date         Jul 2017
 * Author       Djordje Rosic
 *
 * Test configuration AVR :
 *  MCU           :        ATMEGA32
 *  Dev. Board    :        EasyAVR v7
 *  SW            :        mikroC PRO for AVR v7.0.0
 *
 * Description :
 *
 *   This example demonstrates the use of Secure 3 click.
 *
 *   IMPORTANT NOTICE: Some features of this device require config or data
 *   zones of ATSHA204A device to be locked. After a zone is locked, it cannot
 *   be unlocked again. This security measure means it is not possible to
 *   reprogram the device for a different purpose, once it is configured.
 *
 *   This example will only use the functions that won't require locking
 *   any of the data zones, so that any change they make to the device is not
 *   permanent. It also includes lock function, commented out, as a
 *   demonstration on how to lock the device.
 *
 *       Note: Data output is via UART, turn on the required switches on
 *   the EasyAVR board.
 *       All the basic functions for this device are prefixed with atcab_
 *   and are contained in the atca_basic.c file. Some of the functions in the
 *   library are used for other security devices, and won't work on Secure 3
 *   click. Refer to the device datasheet to see which functions are availible.
 *
 ******************************************************************************/

#include "SECURE_driver.c"

uint8_t bufferIn  [ 128 ];
uint8_t bufferOut [ 128 ];

//Sample data to test MAC verification
uint8_t otherData [ 13 ] = {0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
uint8_t checkMac [ 32 ] = {
 0xCD, 0x89, 0xA9, 0xA0, 0xD2, 0x63, 0x44, 0x9F, 0x58, 0x48, 0x81, 0xFC, 0xEF,
 0xF6, 0xD1, 0xFD, 0x82, 0xF7, 0xEF, 0xF3, 0x47, 0x4C, 0xB7, 0x1B, 0x1E, 0x9C,
 0x76, 0x13, 0x02, 0x18, 0x6E, 0x32 };

static void additionalTest ();
static void MACTest ();

void outputHex (uint8_t * pOutput, uint8_t length);

/*
   System Initialization
   Initializes Memory Manager, UART output, I2C connection, and HAL layer.
   Also initializes the ATCADevice object used by the CryptoAuthLib library.
 */
void systemInit()
{
    MM_Init();
    UART1_Init( 9600 );

    TWI_Init( 100000 );
    SECURE_HAL_init( 0x64, TWI_Start, TWI_Stop, TWI_Write, TWI_Read );
    Delay_1sec();

    atcab_init( &cfg_sha204a_i2c_default );
    memset (bufferIn, 0x00, 128);
    memset (bufferOut, 0x00, 128);

    LOG( "\r\n\r\nSystem Initialized\r\n" );
}


void main()
{
    systemInit();

    LOG( "\r\nStarting device testing...\r\n" );

    //First test - Reading serial number
    if (atcab_read_serial_number(bufferOut) == ATCA_SUCCESS)
    {
        LOG( "\r\n Serial number: " );
        outputHex (bufferOut, 9);
    }
    else  LOG( "\r\n Reading serial number failed..." );


    //Second test - Reading device config
    delay_ms (1500);
    memset (bufferOut, 0x00, 128);
    if (atcab_read_config_zone(ATSHA204A, bufferOut) == ATCA_SUCCESS)
    {
        LOG( "\r\n\r\n First 32 bytes of device configuration: " );
        outputHex (bufferOut, 32);
    }
    else  LOG( "\r\n\r\n Reading config zone failed..." );


    //Third test - MAC verification
    delay_ms (2500);
    memset (bufferOut, 0x00, 128);
    memset (bufferIn, 0x12, 128);
    if (atcab_checkmac( 0, 0, bufferIn, checkMac, otherData) == ATCA_SUCCESS)
    {
        LOG( "\r\n\r\n Check MAC successful. " );
    }
    else  LOG( "\r\n\r\n Check MAC failed..." );


    additionalTest ();
    MACTest ();


    LOG( "\r\n\r\n--------------------------------------------" );
    LOG( "\r\nEND" );
}


/*
    This function can lock the config zone, the and generate random number.
    Zone locking is commented out by default, as it will make permanent change
    to the device. While the zone is still unlocked, random number generator
    will output 0xFFFF0000FFFF0000FFFF0000FFFF0000
*/
static void additionalTest ()
{
/*
  IMPORTANT: Uncommenting these lines permanently locks the config zone of the
  device.

  This is irreversible, so make sure configuration is set according to your
  needs. All the following tests won't work unless config zone is locked.


   if (atcab_lock_config_zone(bufferOut) == ATCA_SUCCESS)
       LOG( "\r\n Config zone locked " );
   else  LOG( "\r\n Locking config zone failed or zone is already locked" );
*/

/*
    Fourth test - Random number generator
    Will only return 0xFFFF0000FFFF0000 unless config zone is locked first.
*/
    memset (bufferOut, 0x00, 128);
    if (atcab_random(bufferOut) == ATCA_SUCCESS)
    {
        LOG( "\r\n\r\n Generated random number: " );
        outputHex (bufferOut, 32);
    }
    else  LOG( "\r\n\r\n Generating random number failed..." );

}
/*
    This function creates a MAC for a given input, and then checks if it is
    valid using verify function. Array bufferIn is input for MAC generation
    and verification, and needs to match for the atcab_mac and atcab_checkmac
    commands. It can have any value for atcab_nonce.
*/
static void MACTest ()
{
    //Generates nonce for use in MAC generation
    memset (bufferIn, 0x45, 128);
    if (atcab_nonce(bufferIn) == ATCA_SUCCESS)
    {
        LOG( "\r\n\r\n Nonce generated." );
    }
    else  LOG( "\r\n\r\n Nonce generation failed..." );
    delay_ms (1500);

    //Generates MAC for given input
    memset (bufferOut, 0x00, 128);
    memset (bufferIn, 0x14, 128);
    if (atcab_mac( 0, 0, bufferIn, bufferOut ) == ATCA_SUCCESS)
    {
        LOG( "\r\n\r\n MAC generated: " );
        outputHex (bufferOut, 32);
    }
    else  LOG( "\r\n\r\n Mac generation failed..." );
     delay_ms (1500);

    //Checks if the generated MAC is valid
    memset (bufferIn, 0x14, 128);
    if (atcab_checkmac( 0, 0, bufferIn, bufferOut, otherData) == ATCA_SUCCESS)
    {
        LOG( "\r\n\r\n Check MAC successful. " );
    }
    else  LOG( "\r\n\r\n Check MAC failed..." );
}



//Help function for outputing data in hex format
void outputHex (uint8_t * pOutput, uint8_t length)
{
    uint8_t i;
    char text [10];
    for( i = 0; i < length; i++ )
    {
        ByteToHex( pOutput[ i ], text );
        LOG( Ltrim(text) );
        LOG( " " );
    }
}